"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_line_args_1 = __importDefault(require("command-line-args"));
const cors_1 = __importDefault(require("cors"));
const express_1 = __importDefault(require("express"));
const swagger_ui_express_1 = require("swagger-ui-express");
const typescript_lambda_api_1 = require("typescript-lambda-api");
/**
 * Simple console application that hosts an express HTTP
 * server.
 *
 * A request is mapped from the HTTP request to the ApiServer
 * event format, a ApiServer instance then processes the request.
 * The ApiServer response is mapped to a standard HTTP response and
 * returned to the express client.
 */
class ApiConsoleApp extends typescript_lambda_api_1.ApiApp {
    constructor(controllersPath, appConfig, appContainer) {
        super(controllersPath, appConfig, appContainer);
        this.expressApp = express_1.default();
    }
    async run(request, context) {
        return await this.apiServer.processEvent(request, context);
    }
    /**
     * Starts the express server.
     *
     * @param args Command line arguments for this server, see --help for more info.
     */
    async runServer(args) {
        let self = this;
        let options = this.parseArguments(args);
        let listenOnAllHosts = options.host === "*";
        let baseUrl = `http://${options.host}:${options.port}`;
        await super.initialiseControllers();
        this.expressApp.use(cors_1.default({
            origin: options["cors-origin"]
        }));
        if (this.appConfig.openApi.enabled) {
            this.configureSwaggerUi();
        }
        this.expressApp.all("*", (req, res, next) => self.handleHttpRequest(self, req, res, next));
        this.server = await new Promise((resolve, reject) => {
            try {
                if (listenOnAllHosts) {
                    let server = this.expressApp.listen(options.port, () => resolve(server));
                }
                else {
                    let server = this.expressApp.listen(options.port, options.host, () => resolve(server));
                }
            }
            catch (ex) {
                reject(ex);
            }
        });
        console.log(`App is now listening for HTTP requests on ${baseUrl} ...`);
    }
    configureSwaggerUi() {
        let basePath = this.appConfig.base || "";
        let specUrl = `${basePath}/open-api.json`;
        this.expressApp.use(`${basePath}/swagger`, swagger_ui_express_1.serve, swagger_ui_express_1.setup(null, {
            explorer: true,
            swaggerUrl: specUrl
        }));
    }
    async stopServer() {
        if (!this.server) {
            throw new Error("stopServer can only be called after runServer has been called and has completed");
        }
        await new Promise((resolve, reject) => {
            try {
                this.server.close(() => resolve());
            }
            catch (ex) {
                reject(ex);
            }
        });
    }
    parseArguments(args) {
        return command_line_args_1.default(ApiConsoleApp.APP_OPTIONS, {
            argv: args
        });
    }
    async handleHttpRequest(self, request, response, onError) {
        try {
            let apiRequestEvent = await self.mapRequestToApiEvent(request);
            let apiResponse = await self.run(apiRequestEvent, {});
            self.forwardApiResponse(apiResponse, response);
        }
        catch (ex) {
            onError(ex);
        }
    }
    async mapRequestToApiEvent(request) {
        let apiRequest = new typescript_lambda_api_1.ApiRequest();
        apiRequest.httpMethod = request.method;
        apiRequest.path = request.path;
        Object.keys(request.headers)
            .forEach(h => apiRequest.headers[h] = request.headers[h]);
        Object.keys(request.query)
            .forEach(q => apiRequest.queryStringParameters[q] = request.query[q]);
        // read request body (if any) as a base 64 string
        return await new Promise((resolve, reject) => {
            let body = "";
            request.setEncoding("base64");
            request.on("data", d => body += d);
            request.on("end", () => {
                apiRequest.body = body;
                apiRequest.isBase64Encoded = true;
                resolve(apiRequest);
            });
            request.on("error", reject);
        });
    }
    forwardApiResponse(apiResponse, response) {
        let headers = apiResponse.headers;
        response.status(apiResponse.statusCode);
        Object.keys(headers).forEach(h => response.header(h, headers[h]));
        let body = apiResponse.body;
        if (apiResponse.isBase64Encoded) {
            body = Buffer.from(body, "base64");
        }
        response.send(body);
    }
}
ApiConsoleApp.APP_OPTIONS = [
    { name: "port", alias: "p", type: Number, defaultValue: 8080 },
    { name: "host", alias: "h", type: String, defaultValue: "*" },
    { name: "cors-origin", alias: "c", type: String, defaultValue: "*" }
];
__decorate([
    typescript_lambda_api_1.timed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [typescript_lambda_api_1.ApiRequest, Object]),
    __metadata("design:returntype", Promise)
], ApiConsoleApp.prototype, "run", null);
__decorate([
    typescript_lambda_api_1.timed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [ApiConsoleApp, Object, Object, Function]),
    __metadata("design:returntype", Promise)
], ApiConsoleApp.prototype, "handleHttpRequest", null);
exports.ApiConsoleApp = ApiConsoleApp;
//# sourceMappingURL=ApiConsoleApp.js.map